/**
* The Society of Actuaries (SOA) makes this computer model and its computer
* code available to its members for educational purposes. The model and its
* code may be modified and redistributed without prior authorization from the
* SOA. The SOA distributes this computer model and its computer code in the
* hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
* implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
* PURPOSE.
*/ 

package agents;

//==============================================================================
// Import statements
//------------------------------------------------------------------------------
import java.util.ArrayList;
import java.util.ListIterator;
import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.parameter.Parameter;
import repast.simphony.random.RandomHelper;
import repast.simphony.ui.probe.ProbeID;
import resources.AgentID;
import resources.Constants;
import resources.Constants.IntensityLevel;
import resources.Constants.StageOfChange;
import resources.Constants.WeightCategory;
import resources.Goal;
import resources.GoalParameter;
import resources.Constants.Disease;
import resources.Constants.GoalDescription;

/**
 * @author      Alan Mills
 * <p>
 * This class implements the Employee agent. An individual employee of the
 * Employer. An Employee decides whether to participate in the wellness program,
 * decides whether to comply with the programs exercise recommendations,
 * progresses along stages of change for maintaining an exercise regimen,
 * decides whether to terminate employment in order to work elsewhere, and
 * decides when to retire. The Employee also changes body weight, may develop
 * diabetes, may incur medical expenses for diabetes, has days of absence from
 * work, and has days of presenteeism.
 * @version     1.0
 */
@SuppressWarnings("unused")
public class Employee extends Agent
{
//==============================================================================
// Attribute variables
// These variables describe the agent at a current point in time.
// They are available to other classes through "getter" methods. 
//------------------------------------------------------------------------------
    private int currentAge;
    private Disease currentDiseaseStatus;
    private boolean currentEmploymentStatus;
    private GoalDescription currentEmployeeFirstGoal;
    private boolean currentProgramParticipationStatus;
    private boolean currentRetirementStatus;
    private StageOfChange currentStageOfChange;
    private WeightCategory currentWeightCategory;
    private int employmentEntryAge;
    private int employmentTerminationAge;
    private ArrayList<Goal> employeeGoals = new ArrayList<Goal>();
//==============================================================================

//==============================================================================
// Memory variables
// These variables include changing information over time.
// They represent the agent's memory.
//------------------------------------------------------------------------------

    private Disease previousDiseaseStatus;    
    private boolean previousEmploymentStatus;
    private boolean previousProgramParticipationStatus;
    private boolean previousRetirementStatus;
    private StageOfChange previousStageOfChange;
    private WeightCategory previousWeightCategory;
//==============================================================================

//==============================================================================
// Other variables
// These are other variables used in the class.
// They are not available through "getter" methods.
//------------------------------------------------------------------------------
    private Context<Object> simulationContext;
    private double tickCount;

//==============================================================================


//==============================================================================
// Method:  Employee() constructor
//------------------------------------------------------------------------------
    /**
     * Constructs an Employee agent.
     *
     * The constructor sets the Employee's attribute variables based on input
     * parameters. 
     *   
     * @param pContext The identifier for the simulation context
     * @param pEmployeeID The Employee's identifier.
     */
    public Employee(Context<Object> pContext, AgentID pEmployeeID)
    {
        setAgentID(pEmployeeID);
        simulationContext = pContext;

        init_InitializeVariables_Constructor();
        init_DetermineInitialAge();
        init_DetermineInitialWeightCategory();
        
        // Note that the initial stage of change must be determined after
        // the initial weight category.
        init_DetermineInitialStageOfChange();

        // Note that the initial disease status must be determined after
        // the age, stage of change, and weight category.
        init_DetermineInitialDiseaseStatus();
        init_SetGoalPriorities(init_GetGoalParameters());

    }//end Employee()
//==============================================================================

    
//==============================================================================
// Methods: Initialization methods for the agent. These are used in
//          the agent's constructor method or at the beginning or end of each
//          simulation period.
//==============================================================================

//==============================================================================
// Method:  init_InitializeVariables_Constructor()
//------------------------------------------------------------------------------
    /**
     * Initializes variables (used in the constructor)
     * 
     */
    public void init_InitializeVariables_Constructor()
    {
        currentAge = 0;
        currentDiseaseStatus = Disease.None;
        currentEmployeeFirstGoal = GoalDescription.None;
        currentEmploymentStatus = true;
        currentProgramParticipationStatus = false;
        currentRetirementStatus = false;
        currentStageOfChange = StageOfChange.None;
        currentWeightCategory = WeightCategory.None;
        employmentEntryAge = 0;
        employmentTerminationAge = 0;

        previousDiseaseStatus = Disease.None;
        previousEmploymentStatus = false;
        previousProgramParticipationStatus = false;
        previousRetirementStatus = false;
        previousStageOfChange = StageOfChange.None;
        previousWeightCategory = WeightCategory.None;

        tickCount = 0.0;

    }//end init_InitializeVariables_Constructor()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  init_InitializeVariables_BOY()
//------------------------------------------------------------------------------
    /**
     * Initializes variables at the beginning of each simulation period.
     * 
     */
    public void init_InitializeVariables_BOY()
    {

        // Intentionally blank, until needed.

    }//end init_InitializeVariables_BOY()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  init_DetermineInitialAge()
//------------------------------------------------------------------------------
    /**
     * Determines the Employee's initial age.
     * 
     * As the Employee agent is constructed, this method determines the Employee's
     * initial age, based on parameters supplied by the model user.
     */
    private void init_DetermineInitialAge()
    {

        if (Environment.getEmployeeAgeDistributionType().equals(Constants.NORMAL_DISTRIBUTION))
        {
            RandomHelper.createNormal(Environment.getEmployeeAverageAge(), 
                    Environment.getEmployeeAgeStandardDeviation());
            currentAge = Math.max(Math.min(RandomHelper.getNormal().nextInt(),
                    Constants.DATA_TABLE_MAXIMUM_AGE), Constants.DATA_TABLE_MINIMUM_AGE);
        }//end if
        
        // If the Employee age distribution type is "random", distribute the Employee's
        // age randomly over the age interval covered by the data tables.
        else
        {
            currentAge =
                    RandomHelper.nextIntFromTo(Constants.DATA_TABLE_MINIMUM_AGE, Constants.DATA_TABLE_MAXIMUM_AGE);
        }//end else
        
        employmentEntryAge = currentAge;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This tests how the initial ages are allocated.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_DetermineInitialAge() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Current age:  " + currentAge);
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
    }//end init_DetermineInitialAge()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  init_DetermineInitialDiseaseStatus()
//------------------------------------------------------------------------------    
    /**
     * Determines the Employee's initial disease status.
     * 
     * As the Employee agent is constructed, this method determines the Employee's
     * initial disease status as either None or Diabetes, based on prevalence and 
     * incidence probability parameters supplied by the model user.
     */
    private void init_DetermineInitialDiseaseStatus()
    {
        currentDiseaseStatus = Disease.None;
            
        double randomNumber = RandomHelper.nextDouble();
        
        double tickCount2 = RunEnvironment.getInstance().getCurrentSchedule().getTickCount();
        
        // To create the initial Employee population, use diabetes prevalence rates.
        // Thereafter, use incidence rates.
        if (tickCount2 <= 0.00)
        {
            if (randomNumber <
                    (Environment.getDiabetesPrevalenceRate()
                    [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]) *
                        Environment.getDiabetesPrevalenceAdjustmentFactor())
            {
                currentDiseaseStatus = Disease.Diabetes;
            }//end if
        }//end if
        else
        {
            if (randomNumber <
                    (Environment.getDiabetesIncidenceRate()
                    [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]) *
                         Environment.getDiabetesIncidenceAdjustmentFactor())
            {
                currentDiseaseStatus = Disease.Diabetes;
            }//end if
        }

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This tests how the disease is initialized by printing out to the console
// the relevant data for every Employee.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_DetermineInitialDiseaseStatus() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Current age:  " + currentAge);
//  System.out.println("Current weight category:  " + currentWeightCategory.toString());
//  System.out.println("Current stage of change:  " + currentStageOfChange.toString());
//  System.out.println("Random number:  " + randomNumber);
//  System.out.println("Tick count:  " + tickCount2);
//  System.out.println("Diabetes prevalence probability:  " + Environment.getDiabetesPrevalenceRate()
//                [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]*
//                    Environment.getDiabetesPrevalenceAdjustmentFactor());
//  System.out.println("Diabetes prevalence adjustment factor:  " + 
//                    Environment.getDiabetesPrevalenceAdjustmentFactor());
//  System.out.println("Diabetes incidence adjustment factor:  " + 
//                    Environment.getDiabetesIncidenceAdjustmentFactor());
//  System.out.println("Diabetes incidence probability:  " + Environment.getDiabetesIncidenceRate()
//                [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]*
//                    Environment.getDiabetesIncidenceAdjustmentFactor());
//  System.out.println("Current disease status:  " + currentDiseaseStatus.toString());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    }//end init_DetermineInitialDiseaseStatus()
//------------------------------------------------------------------------------    
     
//==============================================================================
// Method:  init_DetermineInitialWeightCategory()
//------------------------------------------------------------------------------
    /**
     * Determines the Employee's initial weight category.
     * 
     * As the Employee agent is constructed, this method determines the Employee's
     * initial weight category ("Normal", "Overweight", or "Obese"), based on
     * parameters supplied by the model user.
     */
    private void init_DetermineInitialWeightCategory()
    {
        double randomNumber2 = RandomHelper.nextDouble();
        
        if (randomNumber2 <= Environment.getEmployeesNormalWeightPercent()/100)
        {
            currentWeightCategory = WeightCategory.Normal;
        }//end if
        else
        {
            if (randomNumber2 <=
                Environment.getEmployeesNormalWeightPercent()/100 +
                Environment.getEmployeesOverweightPercent()/100)
            {
                currentWeightCategory = WeightCategory.Overweight;
            }
            else
            {
                currentWeightCategory = WeightCategory.Obese;
            }
        }//end else

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This tests how the weight category is assigned.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_DetermineInitialWeightCategory() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Random number:  " + randomNumber2);
//  System.out.println("Normal weight percent:  " +
//          Environment.getEmployeesNormalWeightPercent());
//  System.out.println("Overweight percent:  " +
//          Environment.getEmployeesOverweightPercent());
//  System.out.println("Obese percent:  " +
//          Environment.getEmployeesObesePercent());
//  System.out.println("Current weight category:  " + currentWeightCategory.toString());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
    }//end init_DetermineInitialWeightCategory()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  init_DetermineInitialStageOfChange()
//------------------------------------------------------------------------------
    /**
     * Determines the Employee's initial stage of change.
     * 
     * As the Employee agent is constructed, this method determines the Employee's
     * initial stage of change("Stage 1", "Stage 2", or "Stage 3"), based on
     * parameters supplied by the model user.
     */
    private void init_DetermineInitialStageOfChange()
    {

        double randomNumber1 = RandomHelper.nextDouble();

        switch (getCurrentWeightCategory())
        {
            case Normal:
            {
                if (randomNumber1 <= Environment.getEmployeeStageOfChange_NormalWeight_Stage1Percent()/100)
                {
                    currentStageOfChange = StageOfChange.Stage1;
                    break;
                }//end if
                else
                {
                    if (randomNumber1 <=
                        Environment.getEmployeeStageOfChange_NormalWeight_Stage1Percent()/100 +
                        Environment.getEmployeeStageOfChange_NormalWeight_Stage2Percent()/100)
                    {
                        currentStageOfChange = StageOfChange.Stage2;
                        break;
                    }
                    else
                    {
                        currentStageOfChange = StageOfChange.Stage3;
                        break;
                    }
                }
            }//end case

            case Overweight:
            {
                if (randomNumber1 <= Environment.getEmployeeStageOfChange_Overweight_Stage1Percent()/100)
                {
                    currentStageOfChange = StageOfChange.Stage1;
                    break;
                }//end if
                else
                {
                    if (randomNumber1 <=
                        Environment.getEmployeeStageOfChange_Overweight_Stage1Percent()/100 +
                        Environment.getEmployeeStageOfChange_Overweight_Stage2Percent()/100)
                    {
                        currentStageOfChange = StageOfChange.Stage2;
                        break;
                    }
                    else
                    {
                        currentStageOfChange = StageOfChange.Stage3;
                        break;
                    }
                }
            }//end case           
            case Obese:
            {
                if (randomNumber1 <= Environment.getEmployeeStageOfChange_Obese_Stage1Percent()/100)
                {
                    currentStageOfChange = StageOfChange.Stage1;
                    break;
                }//end if
                else
                {
                    if (randomNumber1 <=
                        Environment.getEmployeeStageOfChange_Obese_Stage1Percent()/100 +
                        Environment.getEmployeeStageOfChange_Obese_Stage2Percent()/100)
                    {
                        currentStageOfChange = StageOfChange.Stage2;
                        break;
                    }
                    else
                    {
                        currentStageOfChange = StageOfChange.Stage3;
                        break;
                    }
                }
            }//end case
            
            default: currentStageOfChange = StageOfChange.Stage1;
            
        }//end switch
        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This tests how the stage of change is assigned.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_DetermineInitialStageOfChange() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Current weight category:  " + currentWeightCategory.toString());
//  System.out.println("Random number:  " + randomNumber1);
//        switch (getCurrentWeightCategory())
//        {
//            case Normal:
//            {
//                System.out.println("Stage 1 percent:  " +
//                        Environment.getEmployeeStageOfChange_NormalWeight_Stage1Percent());
//                System.out.println("Stage 2 percent:  " +
//                        Environment.getEmployeeStageOfChange_NormalWeight_Stage2Percent());
//                System.out.println("Stage 3 percent:  " +
//                        Environment.getEmployeeStageOfChange_NormalWeight_Stage3Percent());
//                break;
//            }//end case
//
//            case Overweight:
//            {
//                System.out.println("Stage 1 percent:  " +
//                        Environment.getEmployeeStageOfChange_Overweight_Stage1Percent());
//                System.out.println("Stage 2 percent:  " +
//                        Environment.getEmployeeStageOfChange_Overweight_Stage2Percent());
//                System.out.println("Stage 3 percent:  " +
//                        Environment.getEmployeeStageOfChange_Overweight_Stage3Percent());
//                break;
//            }//end case
//            
//            case Obese:
//            {
//                System.out.println("Stage 1 percent:  " +
//                        Environment.getEmployeeStageOfChange_Obese_Stage1Percent());
//                System.out.println("Stage 2 percent:  " +
//                        Environment.getEmployeeStageOfChange_Obese_Stage2Percent());
//                System.out.println("Stage 3 percent:  " +
//                        Environment.getEmployeeStageOfChange_Obese_Stage3Percent());
//                break;
//            }//end case
//
//         }//end switch
//        
//  System.out.println("Current stage of change:  " + currentStageOfChange.toString());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
    }//end init_DetermineInitialStageOfChange()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  init_GetGoalParameters()
//------------------------------------------------------------------------------
    private ArrayList<GoalParameter> init_GetGoalParameters()
    {   
        //Variables
        ArrayList<GoalParameter> goalParameterList = new ArrayList<GoalParameter>();
        GoalDescription goalDescription[] = new GoalDescription[3];
        double probability[] = new double[3];

        goalDescription[0] = GoalDescription.Health;
        goalDescription[1] = GoalDescription.Conformance;
        goalDescription[2] = GoalDescription.Income;
        probability[0] = Environment.getEmployeeFirstGoalPercent_Health();
        probability[1] = Environment.getEmployeeFirstGoalPercent_Conformance();
        probability[2] = Environment.getEmployeeFirstGoalPercent_Income();
        
        for(int i=0; i<3; i++)
        {
            GoalParameter goalParameter = new GoalParameter(goalDescription[i], probability[i]);
            goalParameterList.add(goalParameter);

        }//end for
        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// Tests if the goal parameters are properly accessed.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_GetGoalParameters() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Goal parameter 1:  " + goalParameterList.get(0).getGoalDescription() + 
//                  " " + goalParameterList.get(0).getGoalProbability());
//  System.out.println("Goal parameter 2:  " + goalParameterList.get(1).getGoalDescription() + 
//                  " " + goalParameterList.get(1).getGoalProbability());
//  System.out.println("Goal parameter 3:  " + goalParameterList.get(2).getGoalDescription() + 
//                  " " + goalParameterList.get(2).getGoalProbability());               
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
        return(goalParameterList);
        
    }//end init_GetGoalParameters()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  init_SetGoalPriorities(GoalParameters)
//------------------------------------------------------------------------------
    private void init_SetGoalPriorities(ArrayList<GoalParameter> pGoalParameters)
    {   

        //Variables
        Goal goal1 = null;
        Goal goal2 = null;
        Goal goal3 = null;
        
        double  randomNumber1 = RandomHelper.nextDoubleFromTo(0, 1) * 100;
        double  randomNumber2 = RandomHelper.nextDoubleFromTo(0, 1) * 100;
        
        if (randomNumber1 <= pGoalParameters.get(0).getGoalProbability()) 
        {
            setAgentFirstPriorityGoalDescription(pGoalParameters.get(0).getGoalDescription());
        }//end if
        
        else
        {
            if (randomNumber1 <= (pGoalParameters.get(0).getGoalProbability() +
                    pGoalParameters.get(1).getGoalProbability()))
            {
                setAgentFirstPriorityGoalDescription(pGoalParameters.get(1).getGoalDescription());
            }//end if
            else
            {
                setAgentFirstPriorityGoalDescription(pGoalParameters.get(2).getGoalDescription());
            }//end else
        }//end else
            
        switch (getAgentFirstPriorityGoalDescription())
        {
            case Health:
                goal1 = new Goal(pGoalParameters.get(0).getGoalDescription(), 1);
                employeeGoals.add(goal1);
                
                if (randomNumber2 <= 0.50)
                {
                    goal2 = new Goal(pGoalParameters.get(1).getGoalDescription(), 2);
                    employeeGoals.add(goal2);
                    
                    goal3 = new Goal(pGoalParameters.get(2).getGoalDescription(), 3);
                    employeeGoals.add(goal3);
                }//end if
                else
                {
                    goal2 = new Goal(pGoalParameters.get(2).getGoalDescription(), 2);
                    employeeGoals.add(goal2);
                    
                    goal3 = new Goal(pGoalParameters.get(1).getGoalDescription(), 3);
                    employeeGoals.add(goal3);
                }//end else
                break;

            case Conformance:
                goal1 = new Goal(pGoalParameters.get(1).getGoalDescription(), 1);
                employeeGoals.add(goal1);
                
                double  randomNumber3 = RandomHelper.nextDoubleFromTo(0, 1);
                if (randomNumber3 <= 0.50)
                {
                    goal2 = new Goal(pGoalParameters.get(0).getGoalDescription(), 2);
                    employeeGoals.add(goal2);
                    
                    goal3 = new Goal(pGoalParameters.get(2).getGoalDescription(), 3);
                    employeeGoals.add(goal3);
                }//end if
                else
                {
                    goal2 = new Goal(pGoalParameters.get(2).getGoalDescription(), 2);
                    employeeGoals.add(goal2);
                    
                    goal3 = new Goal(pGoalParameters.get(0).getGoalDescription(), 3);
                    employeeGoals.add(goal3);
                }//end else
                break;

            case Income:
                goal1 = new Goal(pGoalParameters.get(2).getGoalDescription(), 1);
                employeeGoals.add(goal1);
                
                double  randomNumber4 = RandomHelper.nextDoubleFromTo(0, 1);
                if (randomNumber4 <= 0.50)
                {
                    goal2 = new Goal(pGoalParameters.get(1).getGoalDescription(), 2);
                    employeeGoals.add(goal2);
                    
                    goal3 = new Goal(pGoalParameters.get(0).getGoalDescription(), 3);
                    employeeGoals.add(goal3);
                }//end if
                else
                {
                    goal2 = new Goal(pGoalParameters.get(0).getGoalDescription(), 2);
                    employeeGoals.add(goal2);
                    
                    goal3 = new Goal(pGoalParameters.get(1).getGoalDescription(), 3);
                    employeeGoals.add(goal3);
                }//end else
                break;

            default:
            	break;

        }//end switch       

        setAgentSecondPriorityGoalDescription(goal2.getGoalDescription());

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// Tests if the priorities are set properly.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_SetGoalPriorities() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Goal parameter 1:  " + pGoalParameters.get(0).getGoalDescription() + 
//                  " " + pGoalParameters.get(0).getGoalProbability());
//  System.out.println("Goal parameter 2:  " + pGoalParameters.get(1).getGoalDescription() + 
//                  " " + pGoalParameters.get(1).getGoalProbability());
//  System.out.println("Goal parameter 3:  " + pGoalParameters.get(2).getGoalDescription() + 
//                  " " + pGoalParameters.get(2).getGoalProbability());
//  System.out.println("Random number 1:  " + randomNumber1);
//  System.out.println("Random number 2:  " + randomNumber2);
//  System.out.println("First priority goal:  " + getAgentFirstPriorityGoalDescription().toString());
//  System.out.println("Goal 1:  " + goal1.getGoalDescription());
//  System.out.println("Goal 2:  " + goal2.getGoalDescription());
//  System.out.println("Goal 3:  " + goal3.getGoalDescription());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
    }//end init_SetGoalPriorites()
    
//==============================================================================
// Method:  init_IncrementVariables_EOY()
//------------------------------------------------------------------------------
    /**
     * Initializes variables at the end of the year.
     * 
     */
    public void init_IncrementVariables_EOY()
    {

        boolean employeeRemovedStatus = false;
        
        if (getCurrentEmploymentStatus() == false && getPreviousEmploymentStatus() == true ||
                getCurrentRetirementStatus() == true && getPreviousRetirementStatus() == false)
        {
            simulationContext.remove(this);
            Environment.getEmployeeList().remove(this);
            employeeRemovedStatus = true;
        }//end if
        
        else currentAge = currentAge + 1;
        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This tests how the initial ages are allocated.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee init_InitializeVariables_BOY() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Employee removed status:  " + employeeRemovedStatus);
//  System.out.println("Current age:  " + currentAge);
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    }//end init_IncrmentVariables_EOY()
//------------------------------------------------------------------------------    
   
//==============================================================================    


//==============================================================================
// Methods:  "Input" methods for the agent. These are described in the
// detailed description of the model. Each method is preceded by "i#".
//==============================================================================

// For this model, there are no such methods.

//==============================================================================

    
//==============================================================================
// Methods:  "Process" methods for the agent. These are described in the
// detailed description of the model. Each method is preceded
// by "p#" to indicate its number according to the numbering scheme used in the
// detailed description.
//==============================================================================        

//==============================================================================
// Method:  p1_MakeParticipationDecision()
// If the Employer offers a wellness program and the Employee is eligible for it,
// the Employee employs the following process to decide whether to participate in the program.
// This decision is repeated each year of the simulation.
// For a detailed description of this process, see the detailed design document.
//------------------------------------------------------------------------------    
    /**
     * Decide whether to participate in the workplace wellness plan.
     */
    public void p1_MakeParticipationDecision()
    {

        boolean favoredParticipationDecision = false;
        
        previousProgramParticipationStatus = currentProgramParticipationStatus;
        
        if (r8_CompareTargetGroup(currentWeightCategory))
        {
            currentEmployeeFirstGoal = r1_DetermineFirstGoal().getGoalDescription();
            
            switch (currentEmployeeFirstGoal)
            {
                case Health:
                {
                    currentProgramParticipationStatus = true;
                    break;
                }//end case Health
                
                case Conformance:
                {
                    favoredParticipationDecision = r2_DetermineFavoredParticipationDecision();
                    currentProgramParticipationStatus = favoredParticipationDecision;
                    break;
                }
                
                case Income:
                {
                    switch (Environment.getProgramDesign_IncentiveIntensity())
                    {
                        case None:
                        {
                            switch (Environment.getProgramDesign_ChoiceArchitectureIntensity())
                            {
                                case None:
                                {
                                    currentProgramParticipationStatus = false;
                                    break;
                                }

                                case Level1:
                                {
                                    currentProgramParticipationStatus = false;
                                    break;
                                }
                                
                                case Level2:
                                {
                                    currentProgramParticipationStatus = true;
                                    break;
                                }
                            }//end switch
                            break;
                        }//end case None
                            
                        case Level1:
                        {
                            switch(r3_GetDiseaseStatus())
                            {
                                case Diabetes:
                                {
                                    currentProgramParticipationStatus = true;
                                    break;
                                }//end case Diabetes
                                
                                case None:
                                {
                                    switch (Environment.getProgramDesign_ChoiceArchitectureIntensity())
                                    {
                                        case None:
                                        {
                                            currentProgramParticipationStatus = false;
                                            break;
                                        }
        
                                        case Level1:
                                        {
                                            currentProgramParticipationStatus = true;
                                            break;
                                        }
                                        
                                        case Level2:
                                        {
                                            currentProgramParticipationStatus = true;
                                            break;
                                        }
                                    }//end switch
                                    break;
                                }//end case None
                                
                            }//end switch
                            break;
                        }//end case Level1
                            
                        case Level2:
                        {
                            currentProgramParticipationStatus = true;
                            break;
                        }//end case Level2
                            
                    }//end switch
                    break;
                    
                }//end case Income

                default:
				    break;
            }//end switch
        }//end if

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console, to test this process.
// To run the test, remove comments from the code below.
// It is best to run the test with a small number of Employee agents.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee p1_MakeParticipationDecision() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Previous participation status:  " + previousProgramParticipationStatus);
//  System.out.println("Current weight category:  " + getCurrentWeightCategory().toString());
//  System.out.println("Employee in target group:  " + r8_CompareTargetGroup(currentWeightCategory));
//    ListIterator<Goal> iterator = employeeGoals.listIterator();
//    Goal goal;
//    while (iterator.hasNext())
//    {
//        goal = iterator.next();
//        System.out.println("Employee goal:  " +
//                goal.getGoalPriority() + " " +
//                goal.getGoalDescription().toString());
//    }//end while
//  System.out.println("First goal description:  " + currentEmployeeFirstGoal.toString());
//  System.out.println("Neighbor's favored decision:  " + favoredParticipationDecision);
//  System.out.println("Incentive level:  " + Environment.getProgramDesign_IncentiveIntensity().toString());
//  System.out.println("Choice architecture level:  " + Environment.getProgramDesign_ChoiceArchitectureIntensity().toString());
//  System.out.println("Disease status:  " + getCurrentDiseaseStatus().toString());
//  System.out.println("Participation decision:  " + getCurrentProgramParticipationStatus());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    }//end p1_MakeParticipationDecision()
//==============================================================================    
    
//==============================================================================
// Method:  p2_MakeComplianceDecision()
// If the Employee is participating in the wellness program, and the Employee is
// not already at stage of change 3 (Implementation), the Employee employs the
// following process to determine whether to follow the wellness programs
// recommendations to exercise (and thereby increase the stage of change by one
// level). After employing this process, if the Employee is at stage of change 3
// (Implementation), the Employee is maintaining an exercise regimen and is thus
// compliant with the program.
// For a detailed description of this process, see the detailed design document. 
//------------------------------------------------------------------------------
    /**
     * Determine whether to comply with the program's exercise recommendations.
     *
     */
    public void p2_MakeComplianceDecision()
    {

        previousStageOfChange = currentStageOfChange;
        
        if (currentProgramParticipationStatus == false) return;
        
        if(currentStageOfChange != StageOfChange.Stage3)
        {
            currentEmployeeFirstGoal = r1_DetermineFirstGoal().getGoalDescription();
            
            switch (currentEmployeeFirstGoal)
            {
                case Health:
                {
                    currentStageOfChange = r5_IncreaseStageOfChange(currentStageOfChange);
                    break;
                }//end case Health
                    
                case Conformance:
                {
                    if (r4_DetermineFavoredStageOfChange() == StageOfChange.Stage3)
                    {
                      currentStageOfChange = r5_IncreaseStageOfChange(currentStageOfChange);
                      break;
                    }
                }//end case Conformance
                    
                case Income:
                {
                    if (Environment.getProgramDesign_IncentiveIntensity() == IntensityLevel.Level2)
                    {
                        currentStageOfChange = r5_IncreaseStageOfChange(currentStageOfChange);
                    }
                    else
                    {
                        if (Environment.getProgramDesign_Type() == IntensityLevel.Level2)
                        {
                            currentStageOfChange = r5_IncreaseStageOfChange(currentStageOfChange);
                        }
                        else
                        {
                            if (r3_GetDiseaseStatus() == Disease.Diabetes)
                            {
                                currentStageOfChange = r5_IncreaseStageOfChange(currentStageOfChange);
                            }
                            else
                            {
                                if(Environment.getProgramDesign_ChoiceArchitectureIntensity() == IntensityLevel.Level1 ||
                                Environment.getProgramDesign_ChoiceArchitectureIntensity() == IntensityLevel.Level2)
                                {
                                    currentStageOfChange = r5_IncreaseStageOfChange(currentStageOfChange);
                                }//end if
                            }//end else
                        }//end else
                    }//end else
                    break;
                }//end case Income

                default:
				    break;
                    
            }//end switch
        }//end if
           

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console, to test this process.
// To run the test, remove comments from the code below.
// It is best to run the test with a small number of Employee agents.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee p2_MakeComplianceDecision() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Previous stage of change:  " + previousStageOfChange.toString());
//  System.out.println("First goal:  " + currentEmployeeFirstGoal);
//  System.out.println("Favored stage of change:  " + r4_DetermineFavoredStageOfChange().toString());
//  System.out.println("Incentive level:  " + Environment.getProgramDesign_IncentiveIntensity());
//  System.out.println("Program type:  " + Environment.getProgramDesign_Type());
//  System.out.println("Disease status:  " + currentDiseaseStatus);
//  System.out.println("Impact of r5_IncreaseStageOfChange():  " +
//          r5_IncreaseStageOfChange(getPreviousStageOfChange()).toString());
//  System.out.println("Change architecture level:  " + Environment.getProgramDesign_ChoiceArchitectureIntensity());
//  System.out.println("Current stage of change:  " + getCurrentStageOfChange());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    }//end p2_MakeComplianceDecision()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  p3_DetermineWeightCategory()
// If the Employees stage of change is Stage 3 (Implementation) and the
// Employees current weight category is either Overweight or Obese, the
// Employees weight category regression is determined according to weight
// regression probability parameters (from the Excel data file).
// If the Employees stage of change is not Stage 3 and the Employees weight
// category is either Normal or Overweight, the Employees weight category
// progression is determined according to the weight progression probability
// parameters (from the Excel data file).
// Otherwise there is no change in the weight category.
//------------------------------------------------------------------------------
    /**
     * Determine the current weight category.
     *
     */
    public void p3_DetermineWeightCategory()
    {

        double weightRegressionRate = 0.0;
        double weightProgressionRate = 0.0;
        double randomNumber3 = 0.0;
        double randomNumber4 = 0.0;

        previousWeightCategory = currentWeightCategory;
        
        if((getCurrentStageOfChange() == StageOfChange.Stage3) && 
                ((getCurrentWeightCategory() == WeightCategory.Overweight) ||
                (getCurrentWeightCategory() == WeightCategory.Obese)))
        {
            randomNumber3 = RandomHelper.nextDouble();
            weightRegressionRate = Environment.getWeightRegressionRate()
                    [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]*
                    Environment.getWeightRegressionAdjustmentFactor();
        
            if (randomNumber3 < weightRegressionRate)
            {
                currentWeightCategory = r6_DecreaseWeightCategory(currentWeightCategory);
            }//end if
        }//end if
        
        if((getCurrentStageOfChange() != StageOfChange.Stage3) && 
                (getCurrentWeightCategory() == WeightCategory.Normal) ||
                (getCurrentWeightCategory() == WeightCategory.Overweight))
        {
            randomNumber4 = RandomHelper.nextDouble();
            weightProgressionRate = Environment.getWeightProgressionRate()
                    [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]*
                    Environment.getWeightProgressionAdjustmentFactor();
        
            if (randomNumber4 < weightProgressionRate)
            {
                currentWeightCategory = r7_IncreaseWeightCategory(currentWeightCategory);
            }//end if
        }//end if

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console, to test this process.
// To run the test, remove comments from the code below.
// It is best to run the test with a small number of Employee agents.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee p3_DetermineWeightCategory() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Stage of change:  " + getCurrentStageOfChange().toString());
//  System.out.println("Previous weight category:  " + previousWeightCategory.toString());
//  System.out.println("Random number 3:  " + randomNumber3);
//  System.out.println("Weight regression rate:  " + weightRegressionRate);
//  System.out.println("Random number 4:  " + randomNumber4);
//  System.out.println("Weight progression rate:  " + weightProgressionRate);
//  System.out.println("Current weight category:  " + getCurrentWeightCategory().toString());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

    }//end p3_DetermineWeightCategory()
//------------------------------------------------------------------------------

//==============================================================================
// Method:  p4_DetermineDiseaseStatus()
// If the Employee does not currently have Diabetes, the Employees disease
// status is determined based on the Employees stage of change and weight
// category and the diabetes incidence parameters (from the Excel data file)
// If the Employee currently has Diabetes, and the Employee is exercising
// (Stage of Change 3), the disease status is determined based on the Employees
// weight category and the diabetes remission probabilities (from the Excel data
// file).
//------------------------------------------------------------------------------
    /**
     * Determine the current disease status.
     *
     */
    public void p4_DetermineDiseaseStatus()
    {

        double diabetesIncidenceRate = 0.0;
        double diabetesRemissionRate = 0.0;
        double randomNumber5 = 0.0;
        double randomNumber6 = 0.0;
        
        previousDiseaseStatus = currentDiseaseStatus;
        
        switch (getCurrentDiseaseStatus())
        {
            case None:
            {
                randomNumber5 = RandomHelper.nextDouble();
                diabetesIncidenceRate = Environment.getDiabetesIncidenceRate()
                        [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]*
                        Environment.getDiabetesIncidenceAdjustmentFactor();
            
                if (randomNumber5 < diabetesIncidenceRate)
                {
                    currentDiseaseStatus = Disease.Diabetes;
                }//end if
                break;
            }//end case None
            
            case Diabetes:
            {
                randomNumber6 = RandomHelper.nextDouble();
                diabetesRemissionRate = Environment.getDiabetesRemissionRate()
                        [currentAge-1][determineDataColumn(currentWeightCategory, currentStageOfChange)]*
                        Environment.getDiabetesRemissionAdjustmentFactor();
            
                if (randomNumber6 < diabetesRemissionRate && (getCurrentStageOfChange() == StageOfChange.Stage3))
                {
                    currentDiseaseStatus = Disease.None;
                }//end if
                break;
            }//end case Diabetes
        }//end switch
        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console, to test this process.
// To run the test, remove comments from the code below.
// It is best to run the test with a small number of Employee agents.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee p4_DetermineDiseaseStatus() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Stage of change:  " + getCurrentStageOfChange().toString());
//  System.out.println("Weight category:  " + getCurrentWeightCategory().toString());
//  System.out.println("Previous disease status:  " + previousDiseaseStatus.toString());
//  System.out.println("Random number 5:  " + randomNumber5);
//  System.out.println("Diabetes incidence rate:  " + diabetesIncidenceRate);
//  System.out.println("Random number 6:  " + randomNumber6);
//  System.out.println("Diabetes remission rate:  " + diabetesRemissionRate);
//  System.out.println("Current disease status:  " + getCurrentDiseaseStatus().toString());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        
    }//end p4_DetermineDiseaseStatus()
//==============================================================================

//==============================================================================
// Method:  p5_TerminateEmployment()
// Based on the Employees goals and the level of the Employers wellness
// program, the probability of the Employees termination is determined according
// to the parameters J (Turnover  Base probability, and Turnover  Additional
// probability). If the Employee terminates, another Employee is hired with
// characteristics determined as for the simulations initial employees.
//------------------------------------------------------------------------------
    /**
     * Decide whether to terminate employment.
      */
    public void p5_TerminateEmployment()
    {
        double randomNumber6 = 0.0;
        double randomNumber7 = 0.0;
        previousEmploymentStatus = currentEmploymentStatus;
        
        if (getCurrentAge() >= Environment.getRetirementAge()) return;
        
        if (getCurrentEmployeeFirstGoal() == GoalDescription.Health &&
                Environment.getProgramDesign_Type() == IntensityLevel.None)
        {
            randomNumber6 = RandomHelper.nextDouble();
            if (randomNumber6 < (Environment.getTurnover_BaseProbability() + 
                    Environment.getTurnover_AdditionalProbability()))
            {
                currentEmploymentStatus = false;
                employmentTerminationAge = currentAge;
                Environment.p2_CreateEmployee(Environment.getCurrentMaximumEmployeeIDNumber(), simulationContext);
            }//end if
         }//end if
        else
        {
            randomNumber7 = RandomHelper.nextDouble();
            if (randomNumber7 < (Environment.getTurnover_BaseProbability()))
            {
                currentEmploymentStatus = false;
                employmentTerminationAge = currentAge;
                Environment.p2_CreateEmployee(Environment.getCurrentMaximumEmployeeIDNumber(), simulationContext);
            }//end if
        }//end else

        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console, to test this process.
// To run the test, remove comments from the code below.
// It is best to run the test with a small number of Employee agents.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee p5_TerminateEmployment() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Tick count:  " + tickCount);
//  System.out.println("Employee age:  " + getCurrentAge());
//  System.out.println("Employee first goal:  " + getCurrentEmployeeFirstGoal());
//  System.out.println("Random number 6:  " + randomNumber6);
//  System.out.println("Random number 7:  " + randomNumber7);
//  System.out.println("Base termination probability:  " + Environment.getTurnover_BaseProbability());
//  System.out.println("Additional termination probability:  " + Environment.getTurnover_AdditionalProbability());
//  System.out.println("Employee employment status:  " + getCurrentEmploymentStatus());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


    }//end p5_TerminateEmployment()
//==============================================================================

//==============================================================================
// Method:  p6_Retire()
// If the Employee is at the retirement age (parameter J3) or older, and the
// Employee has diabetes or is obese, the employee retires. Also, if the Employee
// reaches the maximum age of the data tables, the Employee retires. Otherwise
// the Employee continues to work. If the Employee retires, another Employee is
// hired with characteristics determined as for the simulations initial employees.
//------------------------------------------------------------------------------
    /**
     * Decide whether to retire.
     * 
     */
    public void p6_Retire()
    {
        
        previousRetirementStatus = currentRetirementStatus;
        
        if (getCurrentAge() < Environment.getRetirementAge()) return;
        
        if (getCurrentEmploymentStatus() == true &&
                getCurrentAge() >= Environment.getRetirementAge());
        {
            if (getCurrentAge() >= Constants.DATA_TABLE_MAXIMUM_AGE ||
                    getCurrentDiseaseStatus() == Disease.Diabetes ||
                    getCurrentWeightCategory() == WeightCategory.Obese)
            {
                currentRetirementStatus = true;
                employmentTerminationAge = currentAge;
            }//end if
        }//end if
        
        if (currentRetirementStatus == true)
        {
            Environment.p2_CreateEmployee(Environment.getCurrentMaximumEmployeeIDNumber(), simulationContext);
        }//end if
        
        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console to test the method.
// To run the test, remove comments from the code below.
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("Employee p6_Retire() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  System.out.println("Employee age:  " + getCurrentAge());
//  System.out.println("Employee weight category:  " + getCurrentWeightCategory());
//  System.out.println("Employee disease status:  " + getCurrentDiseaseStatus());
//  System.out.println("Employee retirement status:  " + getCurrentRetirementStatus());
//  System.out.println("--------------------------------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
                
    }//end p6_Retire()
//------------------------------------------------------------------------------



//==============================================================================
// Methods:  "Rule" methods for the agent. These are described in the
// detailed description of the model. Each method is preceded
// by "r#" to indicate its number according to the numbering scheme used in the
// detailed description.
//==============================================================================
    
//==============================================================================
// Method:  r1_DetermineFirstGoal()
// Determine the Employees goal with the highest priority. This rule returns
// the highest-priority goal.
//------------------------------------------------------------------------------
    private Goal r1_DetermineFirstGoal()
    {   
        Goal currentEmployeeGoal;
        
        ListIterator<Goal> iterator = employeeGoals.listIterator();
        
        while (iterator.hasNext())
        {
            currentEmployeeGoal = iterator.next();
            if (currentEmployeeGoal.getGoalPriority() == 1)
            {
                return currentEmployeeGoal;
            }//end if
        }//end while
        
        return null;
        
    }//end r1_DetermineFirstGoal()
//==============================================================================


//==============================================================================
// Method:  r2_DetermineFavoredParticipationDecision()
// Determine the participation decision that most of the Employees workplace
// neighbors selected in the previous year.
//------------------------------------------------------------------------------
    public boolean r2_DetermineFavoredParticipationDecision()
    {

        double distance1;
        int numberOfNeighbors = 0;
        int numberOfParticipatingNeighbors = 0;
        double averageStatus;
        boolean favoredDecision;
        
        // Locate Employee agents within the neighbor radius,
        // and provide the appropriate counts.
        ListIterator<Employee> iterator1 = Environment.getEmployeeList().listIterator();
         
        while (iterator1.hasNext())
        {
            Employee employee = iterator1.next();
            distance1 = distance(getAgentXLocation(), getAgentYLocation(),
                employee.getAgentXLocation(), employee.getAgentYLocation());
            
            //Note that the Employee's own previous participation status will be considered.
            if (distance1 <= Environment.getEmployeeWorkplaceNeighborhoodRadius())
            {
                numberOfNeighbors ++;
                
                if (employee.getPreviousProgramParticipationStatus())
                    numberOfParticipatingNeighbors ++;
            }//end if
        }//end while

        averageStatus = (double) numberOfParticipatingNeighbors/(double) numberOfNeighbors;
        
        if (averageStatus >= 0.50) favoredDecision = true;
        else favoredDecision = false;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console to test the method.
// To run the test, remove comments from the code below.
// It is best to only do this test with only a few Employees (10-25).
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("---------------");
//  System.out.println("Employee r2_DetermineFavoredParticipationDecision() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  ListIterator<Employee> iterator2 = Environment.getEmployeeList().listIterator();
//  while (iterator2.hasNext())
//  {
//      Employee employee = iterator2.next();
//      double distance2 = distance(getAgentXLocation(), getAgentYLocation(),
//                employee.getAgentXLocation(), employee.getAgentYLocation());
//      System.out.println("Other employee:  " + employee.getEmployeeIDString());
//      System.out.println("Distance to other employee:  " + distance2);
//      System.out.println("Previous participation status of other employee:  " +
//              employee.getPreviousProgramParticipationStatus());
//  }//end while
//  
//  System.out.println("Workplace neighborhood radius parameter:  " + Environment.getEmployeeWorkplaceNeighborhoodRadius());
//  System.out.println("Number of neighbors:  " + numberOfNeighbors);
//  System.out.println("Number of participating neighbors:  " + numberOfParticipatingNeighbors);
//  System.out.println("Average status:  " + averageStatus);
//  System.out.println("Favored decision:  " + favoredDecision);
//  System.out.println("---------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        return favoredDecision;

    }//end r2_DetermineFavoredParticipationDecision()
//==============================================================================
    
//==============================================================================
// Method:  r3_GetDiseaseStatus()
// If the Employee has diabetes, the result is Diabetes. Otherwise, it is None.
//------------------------------------------------------------------------------
    private Disease r3_GetDiseaseStatus()
    {

        return currentDiseaseStatus;
        
    }//end r3_GetDiseaseStatus()
//==============================================================================

//==============================================================================
// Method:  r4_DetermineFavoredStageOfChange()
// Determine the stage of change that most of the Employees workplace neighbors
// had in the previous year. Note that in the previous year these neighbors may
// not have been program participants.
//------------------------------------------------------------------------------
    private StageOfChange r4_DetermineFavoredStageOfChange()
    {
        double distance2;
        int stageOfChange1Count = 0;
        int stageOfChange2Count = 0;
        int stageOfChange3Count = 0;
        int numberOfNeighbors = 0;
        StageOfChange favoredStageOfChange;
        
        // Locate Employee agents within the neighbor radius
        // who participated in the Program in the last period,
        // and provide appropriate stage of change counts.
        ListIterator<Employee> iterator2 = Environment.getEmployeeList().listIterator();
         
        while (iterator2.hasNext())
        {
            Employee employee2 = iterator2.next();
            distance2 = distance(getAgentXLocation(), getAgentYLocation(),
                employee2.getAgentXLocation(), employee2.getAgentYLocation());
            
            //Note that the Employee's own previous stage of change will be considered.
            if (distance2 <= Environment.getEmployeeWorkplaceNeighborhoodRadius())
            {
                numberOfNeighbors++;
                switch (employee2.getPreviousStageOfChange())
                {
                    case Stage1:
                    {
                        stageOfChange1Count++;
                        break;
                    }
                    case Stage2:
                    {
                        stageOfChange2Count++;
                        break;
                    }
                    case Stage3:
                    {
                        stageOfChange3Count++;
                        break;
                    }

                    default:
					    break;
                }//end switch

            }//end if
        }//end while

        if (stageOfChange1Count >= stageOfChange2Count && stageOfChange1Count >= stageOfChange3Count)
            favoredStageOfChange = StageOfChange.Stage1;
        else
        {
             if (stageOfChange2Count >= stageOfChange1Count && stageOfChange2Count >= stageOfChange3Count)
                favoredStageOfChange = StageOfChange.Stage2;
             else
             {
                if (stageOfChange3Count >= stageOfChange1Count && stageOfChange3Count >= stageOfChange2Count)
                    favoredStageOfChange = StageOfChange.Stage3;
                else favoredStageOfChange = StageOfChange.Stage1;
             }//end else
         }//end else
        
        // If there are no other neighbors, make sure the stage of change doesn't change.
        if (numberOfNeighbors == 1 || numberOfNeighbors == 0) favoredStageOfChange = currentStageOfChange;
        
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Test 
// This outputs data to the console to test the method.
// To run the test, remove comments from the code below.
// It is best to only do this test with only a few Employees (10-25).
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  System.out.println("---------------");
//  System.out.println("Employee r4_DetermineFavoredStageOfChange() at " +
//          RunEnvironment.getInstance().getCurrentSchedule().getTickCount());
//  System.out.println("Employee ID:  " + getAgentID().getAgentIDString());
//  ListIterator<Employee> iterator3 = Environment.getEmployeeList().listIterator();
//  while (iterator3.hasNext())
//  {
//      Employee employee = iterator3.next();
//      double distance3 = distance(getAgentXLocation(), getAgentYLocation(),
//                employee.getAgentXLocation(), employee.getAgentYLocation());
//      System.out.println("Other employee:  " + employee.getEmployeeIDString());
//      System.out.println("Distance to other employee:  " + distance3);
//      System.out.println("Previous program participation status:  " +
//              employee.getPreviousProgramParticipationStatus());
//      System.out.println("Previous stage of change of other employee:  " +
//              employee.getPreviousStageOfChange().toString());
//  }//end while
//  
//  System.out.println("Workplace neighborhood radius parameter:  " + Environment.getEmployeeWorkplaceNeighborhoodRadius());
//  System.out.println("Stage of change 1 count:  " + stageOfChange1Count);
//  System.out.println("Stage of change 2 count:  " + stageOfChange2Count);
//  System.out.println("Stage of change 3 count:  " + stageOfChange3Count);
//  System.out.println("Favored stage of change:  " + favoredStageOfChange.toString());
//  System.out.println("---------------");
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

        return favoredStageOfChange;

    }//end r4_DetermineFavoredStageOfChange()

//==============================================================================

//==============================================================================
// Method:  r5_DetermineFavoredStageOfChange()
//------------------------------------------------------------------------------
    private StageOfChange r5_IncreaseStageOfChange(StageOfChange pCurrentStageOfChange)
    {
        if (pCurrentStageOfChange == StageOfChange.Stage1) 
            return StageOfChange.Stage2;
        if (pCurrentStageOfChange == StageOfChange.Stage2) 
            return StageOfChange.Stage3;
        return StageOfChange.Stage3;

    }//end r5_IncreaseStageOfChange()
//==============================================================================

//==============================================================================
// Method:  r6_DecreaseWeightCategory()
//------------------------------------------------------------------------------
    private WeightCategory r6_DecreaseWeightCategory(WeightCategory pCurrentWeightCategory)
    {
        if (pCurrentWeightCategory == WeightCategory.Normal) 
            return WeightCategory.Normal;
        if (pCurrentWeightCategory == WeightCategory.Overweight) 
            return WeightCategory.Normal;
        if (pCurrentWeightCategory == WeightCategory.Obese) 
            return WeightCategory.Overweight;
        return WeightCategory.Normal;

    }//end r6_DecreaseWeightCategory()
//==============================================================================

//==============================================================================
// Method:  r7_IncreaseWeightCategory()
//------------------------------------------------------------------------------    
    private WeightCategory r7_IncreaseWeightCategory(WeightCategory pCurrentWeightCategory)
    {
        if (pCurrentWeightCategory == WeightCategory.Normal) 
            return WeightCategory.Overweight;
        if (pCurrentWeightCategory == WeightCategory.Overweight) 
            return WeightCategory.Obese;
        if (pCurrentWeightCategory == WeightCategory.Obese) 
            return WeightCategory.Obese;
        return WeightCategory.Obese;

    }//end r7_IncreaseWeightCategory()
//==============================================================================    

//==============================================================================
// Method:  r8_CompareTargetGroup()
//------------------------------------------------------------------------------
    private boolean r8_CompareTargetGroup(WeightCategory pCurrentWeightCategory)
    {

        switch (pCurrentWeightCategory)
        {
            case Normal:
            {
                if ((Environment.getProgramDesign_TargetGroup().contentEquals("Normal") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("Normal and Overweight") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("All"))) 
                    return true;
                else return false;
             }//end case Normal
            
            case Overweight:
            {
                if ((Environment.getProgramDesign_TargetGroup().contentEquals("Overweight") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("Normal and Overweight") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("Overweight and Obese") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("All"))) 
                    return true;
                else return false;
             }//end case Overweight
            
            case Obese:
            {
                if ((Environment.getProgramDesign_TargetGroup().contentEquals("Obese") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("Overweight and Obese") ||
                     Environment.getProgramDesign_TargetGroup().contentEquals("All"))) 
                    return true;
                else return false;
             }//end case Obese
            
            default: return false;
            
        }//end switch

    }//end r8_CompareTargetGroup()
//==============================================================================
    
//==============================================================================
// Methods: "Probe" methods for the model user to probe the agent's data.
// Note:    As a simulation runs, the model user can probe an agent by
//          double-clicking on its icon in the display. For more information
//          about probes, see the Repast Simphony reference documentation.
//==============================================================================

    // The "ProbeID" annotation provides the probe ID
    /**
     * Returns the Employee's identification number as a string.
     *
     * @return the Employee's identification number as a string.
     */
    @ProbeID
    public String getEmployeeIDString()
    {
        return (getAgentID().getAgentIDString());
    }//end getEmployeeIDString()

    // The "Parameter" annotation provides the data items to be displayed when
    // a user probes an Employee during a simulation.
    
     /**
     * Returns the Employee's current age.
     *
     * @return the Employee's current age.
     */
    @Parameter (usageName="currentAge", displayName="1. Age")
     public String getStringCurrentAge()
    {
        return Integer.toString(currentAge);
    }

     /**
     * Returns the number of years of the Employee's career.
     *
     * @return the Employee's career in years.
     */
    @Parameter (usageName="currentCareer", displayName="2. Years of career")
     public String getCurrentCareer()
    {
        return Integer.toString(currentAge - employmentEntryAge);
    }
    
    /**
     * Returns the Employee's first goal as a string.
     *
     * @return A string with the Employee's first goal.
     */
    @Parameter (usageName="employeeGoal1", displayName="3a. Goal 1")
    public String getEmployeeGoal1()
    {
        return (getEmployeeGoals().get(0).getGoalDescription().toString());
        
    }//end getEmployeeGoal1()
    
    /**
     * Returns the Employee's second goal as a string.
     *
     * @return A string with the Employees's second goal
     */
    @Parameter (usageName="personGoal2", displayName="3b. Goal 2")
    public String getEmployeeGoal2()
    {
        return (getEmployeeGoals().get(1).getGoalDescription().toString());
        
    }//end getEmployeeGoal2() 

    /**
     * Returns the Employee's third goal as a string.
     *
     * @return A string with the Employee's third goal
     */
    @Parameter (usageName="employeeGoal3", displayName="3c. Goal 3")
    public String getEmployeeGoal3()
    {
        return (getEmployeeGoals().get(2).getGoalDescription().toString());
        
    }//end getEmployeeGoal3() 

    @Parameter (usageName="stringCurrentWeightCategory", displayName="4a. Weight category")
    public String getStringCurrentWeightCategory()
    {
        return currentWeightCategory.toString();
    }

    @Parameter (usageName="intCurrentWeightCategory", displayName="4b. Weight history")
    public int getIntCurrentWeightCategory()
    {

        switch (currentWeightCategory)
        {
            case Normal:
            {
                return 1;
            }
            case Overweight:
            {
                return 2;
            }
            case Obese:
            {
                return 3;
            }

            default:
			    break;
        }
        
        return 0;
    }
    
     /**
     * Returns the Employee's current disease status as a string.
     *
     * @return the Employee's current disease status as a string.
     */
    @Parameter (usageName="stringCurrentDiseaseStatus", displayName="5a. Disease status")
    public String getStringCurrentDiseaseStatus()
    {
        return (currentDiseaseStatus.toString());
    }

    @Parameter (usageName="intCurrentDiseaseStatus", displayName="5b. Disease history")
    public int getIntCurrentDiseaseStatus()
    {
        switch (currentDiseaseStatus)
        {
            case None:
            {
                return 0;
            }
            case Diabetes:
            {
                return 1;
            }
        }
        
        return 0;
    }

    
     /**
     * Returns the Employee's current stage of change as an integer.
     *
     * @return the Employee's current stage of change as an integer.
     */
    @Parameter (usageName="intCurrentStageOfChange", displayName="6. Stage of change")
    public int getIntCurrentStageOfChange()
    {
        if (currentStageOfChange == StageOfChange.Stage1)
            return 1;
        if (currentStageOfChange == StageOfChange.Stage2)
            return 2;
        if (currentStageOfChange == StageOfChange.Stage3)
            return 3;
        return 0;
    }
    @Parameter (usageName="currentParticipationStatus", displayName="7. Participation status")
    public boolean getCurrentProgramParticipationStatus()
    {
        return currentProgramParticipationStatus;
    }
    
    @Parameter (usageName="intCurrentParticipationStatus", displayName="7b. Participation history")
    public int getIntCurrentParticipationHistory()
    {
        if (currentProgramParticipationStatus == false)
                return 0;
        else return 1;
    }
    
    @Parameter (usageName="currentEmploymentStatus", displayName="8. Employment status")
    public boolean getCurrentEmploymentStatus()
    {
        return currentEmploymentStatus;
    }
    
    @Parameter (usageName="currentRetirementStatus", displayName="9. Retirement status")
    public boolean getCurrentRetirementStatus()
    {
        return currentRetirementStatus;
    }
    
     /**
     * Returns the Employee's disease status as an integer.
     *
     * @return the Employee's disease status as an integer.
     */
    public int getIntegerCurrentDiseaseStatus()
    {
        if (currentDiseaseStatus == Disease.None) return (0);
        if (currentDiseaseStatus == Disease.Diabetes) return (1);
         return(0); //This line should never be reached
    
    }//end getIntegerCurrentDiseaseStatus()

    /**
     * Returns the Employee's identification number as an integer.
     *
     * @return the Employees's identification number as an integer.
     */
    public int getEmployeeIDNumber()
    {
        return (getAgentID().getAgentNumber());
    
    }//end getEmployeeID()
    

    
//==============================================================================


//==============================================================================
// Method:  determineDataColumn()
//------------------------------------------------------------------------------
    private int determineDataColumn(WeightCategory pWeightCategory, StageOfChange pStageOfChange)
    {
            switch (pWeightCategory)
            {
                case Normal:
                {
                    switch (pStageOfChange)
                    {
                        case Stage2: case Stage1:
                        {
                            return Constants.NORMAL_WITHOUT_EXERCISE;
                        }

                        case Stage3:
                        {
                            return Constants.NORMAL_WITH_EXERCISE;
                        }

                        default:
                        {
                            return Constants.NORMAL_WITHOUT_EXERCISE;
                        }
                    }//end switch
                }//end case

                case Overweight:
                {
                    switch (pStageOfChange)
                    {
                        case Stage2: case Stage1:
                        {
                            return Constants.OVERWEIGHT_WITHOUT_EXERCISE;
                        }

                        case Stage3:
                        {
                            return  Constants.OVERWEIGHT_WITH_EXERCISE;
                        }

                        default:
                        {
                            return  Constants.OVERWEIGHT_WITHOUT_EXERCISE;
                         }
                    }//end switch
                }//end case

                case Obese:
                {
                    switch (pStageOfChange)
                    {
                        case Stage2: case Stage1:
                        {
                            return  Constants.OBESE_WITHOUT_EXERCISE;
                        }

                        case Stage3:
                        {
                            return  Constants.OBESE_WITH_EXERCISE;

                        }

                        default:
                        {
                            return  Constants.OBESE_WITHOUT_EXERCISE;
                        }
                    }//end switch
                }//end case

                default:
				    break;
            }//end switch
            
            return 0;

    }//end determineDataColumn()



//==============================================================================
// Methods:  "Analysis" methods to support simulation charts and graphs.
//------------------------------------------------------------------------------
    /**
     * Counts the number of Employees.
     * 
     * This method is used to generate several charts.
     *  
     * @return 1 for each Employee
     */
    public int analysis_GetEmployeeCount_Total()
    {
        if (getCurrentEmploymentStatus() == true &&
                getCurrentRetirementStatus() == false)
        {
            return (1);
        }
        else return 0;
         
    }//end analysis_GetEmployeeCount()

    /**
     * Counts the number of Employees who terminate.
     * 
     *  
     * @return 1 for each Employee who terminates
     */
    public int analysis_GetEmployeeCount_Termination()
    {
        if (getCurrentEmploymentStatus() == false)
        {
            return (1);
        }
        else return 0;
         
    }//end analysis_GetEmployeeCount_Termination()


    /**
     * Counts the number of Employees who retire.
     * 
     *  
     * @return 1 for each Employee who terminates
     */
    public int analysis_GetEmployeeCount_Retirement()
    {
        if (getCurrentRetirementStatus() == true)
        {
            return (1);
        }
        else return 0;
         
    }//end analysis_GetEmployeeCount_Retirement()

   /**
     * Counts the number of Employees who have the goal "Health".
     *  
     * @return 1 if the Employee has goal "Health", and 0 otherwise.
     */
    public int analysis_GetEmployeeCount_GoalHealth()
    {
        if (getEmployeeGoal1() == GoalDescription.Health.toString())
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_GoalHealth()
    
   /**
     * Counts the number of Employees who have the goal "Conformance".
     *  
     * @return 1 if the Employee has goal "Conformance", and 0 otherwise.
     */
    public int analysis_GetEmployeeCount_GoalConformance()
    {
        if (getEmployeeGoal1() == GoalDescription.Conformance.toString())
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_GoalConformance()
    
   /**
     * Counts the number of Employees who have the goal "Income".
     *  
     * @return 1 if the Employee has goal "Income", and 0 otherwise.
     */
    public int analysis_GetEmployeeCount_GoalIncome()
    {
        if (getEmployeeGoal1() == GoalDescription.Income.toString())
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_GoalIncome()
    
    
    /**
     * Gets the age of each employee.
     * 
     *  
     * @return the age of each employee
     */
    public int analysis_GetAge_Total()
    {

            return (getCurrentAge());
         
    }//end analysis_GetAge_Total()

   /**
     * Gets the age of each participant.
     * 
     *  
     * @return the age of each participant
     */
    public int analysis_GetAge_Participating()
    {

        if (getCurrentProgramParticipationStatus() == true)
        {
            return (getCurrentAge());
        }
        else return 1;
         
    }//end analysis_GetAge_Participating()

    /**
     * Gets the average age of each employee.
     * 
     *  
     * @return the average age of each employee
     */
    public int analysis_GetAge_Average_Total()
    {

            return (getCurrentAge()/analysis_GetEmployeeCount_Aggregate());
         
    }//end analysis_GetAge_Average_Total()
    
   /**
     * Gets the average age of each participant.
     * 
     *  
     * @return the average age of each participant
     */
    public int analysis_GetAge_Average_Participating()
    {

        if (getCurrentProgramParticipationStatus() == true)
        {
            return (getCurrentAge()/analysis_GetParticipationCount_Aggregate());
        }
        else return 0;
         
    }//end analysis_GetAge_Average_Participating()

   /**
     * Gets the average age of each non-participant.
     * 
     *  
     * @return the average age of each non-participant
     */
    public int analysis_GetAge_Average_NonParticipating()
    {

        if (getCurrentProgramParticipationStatus() == false)
        {
            return (getCurrentAge()/analysis_GetNonParticipationCount_Aggregate());
        }
        else return 0;
         
    }//end analysis_GetAge_Average_NonParticipating()

    
   /**
     * Gets the age of each person with diabetes.
     * 
     *  
     * @return the age of each person with diabetes
     */
    public int analysis_GetAge_Diabetes()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes)
        {
            return (getCurrentAge());
        }
        else return 1;
         
    }//end analysis_GetAge_Diabetes()
    
    
    /**
     * Counts the total number of Employees with Diabetes.
     * 
     * This method is used to generate the simulation's "Disease prevalence" graph.
     *  
     * @return 1 if the Employee has Diabetes and 0 otherwise.
     */
    public int analysis_GetDiabetesCount_Total()
    {
        if (currentDiseaseStatus == Disease.Diabetes)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else
        
    }//end analysis_GetDiabetesCount_Total()

    /**
     * Counts the total number of participating Employees with Diabetes.
     * 
     * This method is used to generate the simulation's "Disease prevalence" graph.
     *  
     * @return 1 if the Employee participates in the Wellness program and has Diabetes, and 0 otherwise.
     */
    public int analysis_GetDiabetesCount_Participating()
    {
        if (currentDiseaseStatus == Disease.Diabetes &&
                (currentProgramParticipationStatus == true))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else
        
    }//end analysis_GetDiabetesCount_Participating()

    /**
     * Counts the total number of non-participating Employees with Diabetes.
     * 
     * This method is used to generate the simulation's "Disease prevalence" graph.
     *  
     * @return 1 if the Employee does not participate in the Wellness program and has Diabetes, and 0 otherwise.
     */
    public int analysis_GetDiabetesCount_NonParticipating()
    {
        if (currentDiseaseStatus == Disease.Diabetes &&
                (currentProgramParticipationStatus == false))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else
        
    }//end analysis_GetDiabetesCount_NoneParticipating()
    
    /**
     * Counts the number of Employees participating in the wellness program.
     * 
     * @return 1 if the Employee participates, and 0 otherwise.
     */
    public int analysis_GetParticipationCount_Total()
    {
        if (currentProgramParticipationStatus == true)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetProgramPartiipationCount_Total()

    /**
     * Counts the number of Employees not participating in the wellness program.
     * 
     * @return 1 if the Employee does not participate, and 0 otherwise.
     */
    public int analysis_GetNonParticipationCount_Total()
    {
        if (currentProgramParticipationStatus == false)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetProgramNonPartiipationCount_Total()

    
    /**
     * Counts the number of Employees participating in the wellness program
     * who have the goal "Health".
     * 
     * This method is used to generate the simulation's "Number of program participants" graph.
     *  
     * @return 1 if the Employee participates and has goal "Health", and 0 otherwise.
     */
    public int analysis_GetParticipationCount_GoalHealth()
    {
        if (currentProgramParticipationStatus == true &&
                (getEmployeeGoal1() == GoalDescription.Health.toString()))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_GoalHealth()
    
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who have the goal "Conformance".
     * 
     * This method is used to generate the simulation's "Number of program participants" graph.
     *  
     * @return 1 if the Employee participates and has goal "Conformance", and 0 otherwise.
     */
    public int analysis_GetParticipationCount_GoalConformance()
    {
        if (currentProgramParticipationStatus == true &&
                (getEmployeeGoal1() == GoalDescription.Conformance.toString()))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_GoalConformance()
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who have the goal "Income".
     * 
     * This method is used to generate the simulation's "Number of program participants" graph.
     *  
     * @return 1 if the Employee participates and has goal "Income", and 0 otherwise.
     */
    public int analysis_GetParticipationCount_GoalIncome()
    {
        if (currentProgramParticipationStatus == true &&
                (getEmployeeGoal1() == GoalDescription.Income.toString()))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_GoalIncome()
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who have normal weight.
     *  
     * @return 1 if the Employee participates and has normal weight.
     */
    public int analysis_GetParticipationCount_WeightNormal()
    {
        if (currentProgramParticipationStatus == true &&
                (getCurrentWeightCategory() == WeightCategory.Normal))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_WeightNormal()
    

    
    /**
     * Counts the number of Employees participating in the wellness program
     * who are overweight.
     * 
     * @return 1 if the Employee participates and is overweight.
     */
    public int analysis_GetParticipationCount_WeightOverweight()
    {
        if (currentProgramParticipationStatus == true &&
                (getCurrentWeightCategory() == WeightCategory.Overweight))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_WeightOverweight()
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who are obese.
     *  
     * @return 1 if the Employee participates and is obese.
     */
    public int analysis_GetParticipationCount_WeightObese()
    {
        if (currentProgramParticipationStatus == true &&
                (getCurrentWeightCategory() == WeightCategory.Obese))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_WeightObese()

    
    
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who are at stage of change 1.
     *  
     * @return 1 if the Employee is participating and is at stage of change 1.
     */
    public int analysis_GetParticipationCount_Stage1()
    {
        if (currentProgramParticipationStatus == true &&
                (getCurrentStageOfChange() == StageOfChange.Stage1))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_Stage1()
    
    
     /**
     * Counts the number of Employees participating in the wellness program
     * who are at stage of change 2.
     *  
     * @return 1 if the Employee is participating and is at stage of change 2.
     */
    public int analysis_GetParticipationCount_Stage2()
    {
        if (currentProgramParticipationStatus == true &&
                (getCurrentStageOfChange() == StageOfChange.Stage2))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_Stage2()
    
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who are at stage of change 3.
     *  
     * @return 1 if the Employee is participating and is at stage of change 3.
     */
    public int analysis_GetParticipationCount_Stage3()
    {
        if (currentProgramParticipationStatus == true &&
                (getCurrentStageOfChange() == StageOfChange.Stage3))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_Stage3()
    
    /**
     * Counts the number of Employees who are at stage of change 1.
     *  
     * @return 1 if the Employee is participating and is at stage of change 1.
     */
    public int analysis_GetEmployeeCount_Stage1()
    {
        if (getCurrentStageOfChange() == StageOfChange.Stage1)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_Stage1()
    
    /**
     * Counts the number of Employees who are at stage of change 2.
     *  
     * @return 1 if the Employee is participating and is at stage of change 2.
     */
    public int analysis_GetEmployeeCount_Stage2()
    {
        if (getCurrentStageOfChange() == StageOfChange.Stage2)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_Stage2()
    
    /**
     * Counts the number of Employees who are at stage of change 3.
     *  
     * @return 1 if the Employee is participating and is at stage of change 3.
     */
    public int analysis_GetEmployeeCount_Stage3()
    {
        if (getCurrentStageOfChange() == StageOfChange.Stage3)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_Stage3()
    
    /**
     * Counts the number of Employees who have normal weight.
     *  
     * @return 1 if the Employee has normal weight.
     */
    public int analysis_GetEmployeeCount_WeightNormal()
    {
        if (getCurrentWeightCategory() == WeightCategory.Normal)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_WeightNormal()
    
    /**
     * Counts the number of Employees who are overweight.
     *  
     * @return 1 if the Employee is overweight.
     */
    public int analysis_GetEmployeeCount_WeightOverweight()
    {
        if (getCurrentWeightCategory() == WeightCategory.Overweight)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_WeightOverweight()
    
    /**
     * Counts the number of Employees who are obese.
     *  
     * @return 1 if the Employee is obese.
     */
    public int analysis_GetEmployeeCount_WeightObese()
    {
        if (getCurrentWeightCategory() == WeightCategory.Obese)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_WeightObese()
    
    
    /**
     * Counts the number of Employees participating in the wellness program
     * who have diabetes.
     * 
     * @return 1 if the Employee participates and has diabetes, and 0 otherwise.
     */
    public int analysis_GetParticipationCount_Diabetes()
    {

        if (currentProgramParticipationStatus == true &&
                (getCurrentDiseaseStatus() == Disease.Diabetes))
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetPartiipationCount_Diabetes()

    /**
     * Counts the number of Employees who have diabetes.
     * 
     * @return 1 if the Employee has diabetes, and 0 otherwise.
     */
    public int analysis_GetEmployeeCount_Diabetes()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes)
        {
            return (1);
        }//end if
        else
        {
            return (0);
        }//end else

    }//end analysis_GetEmployeeCount_Diabetes()
    
    /**
     * Provides the Employee's stage of change as an integer (1, 2, or 3).
     * 
     * This method is used to generate the simulation's "Average stage of change" graph.
     *  
     * @return 1 if the Employee's stage of change is Stage 1, 2 if Stage 2, and 3 if Stage 3.
      */
    public int analysis_GetStageOfChange()
    {
        switch (currentStageOfChange)
        {
            case Stage1:
            {
                return 1;
            }
            case Stage2:
            {
                return 2;
            }
            case Stage3:
            {
                return 3;
            }
            default:
            {
                return 0;
            }

        }//end switch

    }//end analysis_GetStageOfChange()
    
    
     /**
     * Provides the Employee's weight category as a BMI measure.
     * 
     * This method is used to generate the simulation's "Average BMI" graph.
     *  
     * @return BMI_1 if the Employee's weight category is Normal, BMI_2 if overweight
     * and BMI_3 if obese.
      */
    public double analysis_GetBMI_Total()
    {
        switch (currentWeightCategory)
        {
            case Normal:
            {
                return Constants.BMI_NORMAL_WEIGHT;
            }
            case Overweight:
            {
                return Constants.BMI_OVERWEIGHT;
            }
            case Obese:
            {
                return Constants.BMI_OBESE;
            }
            default:
            {
                return 0;
            }

        }//end switch

    }//end analysis_GetBMI()
    
    /**
     * Computes each Employee's career length
     *  
     * @return each Employee's career length.
     */
    public double analysis_GetCareerLength()
    {

        return getCurrentAge() - getEmploymentEntryAge();
        
    }

    
    /**
     * Computes each Employee's average career length
     *  
     * @return each Employee's average career length.
     */
    public double analysis_GetCareerLength_Average_Total()
    {

//        System.out.println("Total employee count:  " + analysis_GetTotalEmployeeCount());
//        System.out.println("Career length:  " + (getCurrentAge() - getEmploymentEntryAge()));
        
        if (analysis_GetEmployeeCount_Aggregate() == 0)
        {
            return 0.0;
        }
        else return (getCurrentAge() - getEmploymentEntryAge())/analysis_GetEmployeeCount_Aggregate();


        
    }//end analysis_GetCareerLength_Average_Total()

    /**
     * Computes the average career length for program participants
     *  
     * @return the average career length for program participants.
     */
    public double analysis_GetCareerLength_Average_Participating()
    {

        if (analysis_GetParticipationCount_Aggregate() == 0)
        {
            return 0.0;
        }
        else return (getCurrentAge() - getEmploymentEntryAge())/
                analysis_GetParticipationCount_Aggregate();

    }//end analysis_GetCareerLength_Average_Participating()
    
    /**
     * Computes the average career length for program non-participants
     *  
     * @return the average career length for program non-participants.
     */
    public double analysis_GetCareerLength_Average_NonParticipating()
    {
        if ((analysis_GetEmployeeCount_Aggregate()- analysis_GetParticipationCount_Aggregate())== 0)
        {
            return 0.0;
        }
        else return (getCurrentAge() - getEmploymentEntryAge())/
                (analysis_GetEmployeeCount_Aggregate()- analysis_GetParticipationCount_Aggregate());

    }//end analysis_GetCareerLength_Average_NonParticipating()


    /**
     * Computes the number of Employees
     * 
     * @return the number of Employees.
     */
    public int analysis_GetEmployeeCount_Aggregate()
    {
        
        return Environment.getEmployeeList().size();

    }//end analysis_GetEmployeeCount_Aggregate()

    
    /**
     * Computes the number of participating employees
     *  
     * @return the number of participating employees.
     */
    public int analysis_GetParticipationCount_Aggregate()
    {
        int participatingEmployeeCount = 0;
        
        ListIterator<Employee> iterator = Environment.getEmployeeList().listIterator();
         
        while (iterator.hasNext())
        {
            Employee employee = iterator.next();
            if (employee.getCurrentProgramParticipationStatus() == true)
                participatingEmployeeCount ++;
        }//end while
                
        return participatingEmployeeCount;

    }//end analysis_GetParticipationCount_Aggregate()

    /**
     * Computes the number of non-participating employees
     *  
     * @return the number of non-participating employees.
     */
    public int analysis_GetNonParticipationCount_Aggregate()
    {
        int nonParticipatingEmployeeCount = 0;
        
        ListIterator<Employee> iterator = Environment.getEmployeeList().listIterator();
         
        while (iterator.hasNext())
        {
            Employee employee = iterator.next();
            if (employee.getCurrentProgramParticipationStatus() == false)
                nonParticipatingEmployeeCount ++;
        }//end while
                
        return nonParticipatingEmployeeCount;

    }//end analysis_GetNonParticipationCount_Aggregate()

    
    /**
     * Computes the annual medical expenditures for diabetes care
     * for all employees.
     *  
     * @return the annual medical expenditures for diabetes care
     * for all employees.
     */
    public double analysis_GetMedicalExpenditures_Total()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes)
        {
            return Environment.getAverageAnnualDiabetesMedicalExpenditure();
        }
        else return 0.0;

    }//end analysis_GetMedicalExpenditures_Total()
    

    /**
     * Computes the annual medical expenditures for diabetes care
     * for participating employees.
     *  
     * @return the annual medical expenditures for diabetes care
     * for participating employees.
     */
    public double analysis_GetMedicalExpenditures_Participating()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                getCurrentProgramParticipationStatus() == true)
        {
            return Environment.getAverageAnnualDiabetesMedicalExpenditure();
        }
        else return 0.0;

    }//end analysis_GetMedicalExpenditures_Participating()
    

    /**
     * Computes the annual medical expenditures for diabetes care
     * for non-participating employees.
     *  
     * @return the annual medical expenditures for diabetes care
     * for participating employees.
     */
    public double analysis_GetMedicalExpenditures_NonParticipating()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                getCurrentProgramParticipationStatus() == false)
        {
            return Environment.getAverageAnnualDiabetesMedicalExpenditure();
        }
        else return 0.0;

    }//end analysis_GetMedicalExpenditures_NonParticipating()
    
    /**
     * Computes the annual average medical expenditures for diabetes care
     * for all employees.
     *  
     * @return the annual medical expenditures for diabetes care
     * for all employees.
     */
    public double analysis_GetMedicalExpenditures_Average_Total()
    {

        if (analysis_GetEmployeeCount_Aggregate()== 0)
        {
            return 0.0;
        }//end if
        else
        {
            if (getCurrentDiseaseStatus() == Disease.Diabetes)
            {
                return Environment.getAverageAnnualDiabetesMedicalExpenditure()/
                    analysis_GetEmployeeCount_Aggregate();
            }
            else return 0.0;
        }//end else

    }//end analysis_GetMedicalExpenditures_Average_Participating()
    
    /**
     * Computes the annual average medical expenditures for diabetes care
     * for participating employees.
     *  
     * @return the annual medical expenditures for diabetes care
     * for participating employees.
     */
    public double analysis_GetMedicalExpenditures_Average_Participating()
    {

        if (analysis_GetParticipationCount_Aggregate()== 0)
        {
            return 0.0;
        }//end if
        else
        {
            if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                    getCurrentProgramParticipationStatus() == true)
            {
                return Environment.getAverageAnnualDiabetesMedicalExpenditure()/
                    analysis_GetParticipationCount_Aggregate();
            }
            else return 0.0;
        }//end else

    }//end analysis_GetMedicalExpenditures_Average_Participating()
    
    /**
     * Computes the annual average medical expenditures for diabetes care
     * for non-participating employees.
     *  
     * @return the annual medical expenditures for diabetes care
     * for non-participating employees.
     */
    public double analysis_GetMedicalExpenditures_Average_NonParticipating()
    {

        if (analysis_GetEmployeeCount_Aggregate() - analysis_GetParticipationCount_Aggregate()== 0)
        {
            return 0.0;
        }//end if
        else
        {
            if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                    getCurrentProgramParticipationStatus() == false)
            {
                return Environment.getAverageAnnualDiabetesMedicalExpenditure()/
                    (analysis_GetEmployeeCount_Aggregate() - analysis_GetParticipationCount_Aggregate());
            }
            else return 0.0;
        }//end else

    }//end analysis_GetMedicalExpenditures_Average_NonParticipating()
    
    /**
     * Computes the annual absence due to diabetes
     * for all employees.
     *  
     * @return the annual absence due to diabetes
     * for all employees.
     */
    public double analysis_GetAbsences_Total()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes)
        {
            return (Environment.getAbsenteeismDays() + Environment.getPresenteeismDays());
        }
        else return 0.0;

    }//end analysis_GetAbsences_Total()
    

    /**
     * Computes the annual absence due to diabetes
     * for participating employees.
     *  
     * @return the annual absence due to diabetes
     * for participating employees.
     */
    public double analysis_GetAbsences_Participating()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                getCurrentProgramParticipationStatus() == true)
        {
            return (Environment.getAbsenteeismDays() + Environment.getPresenteeismDays());
        }
        else return 0.0;

    }//end analysis_GetAbsences_Participating()
    

    /**
     * Computes the annual absence due to diabetes
     * for non-participating employees.
     *  
     * @return the annual absence due to diabetes
     * for participating employees.
     */
    public double analysis_GetAbsences_NonParticipating()
    {

        if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                getCurrentProgramParticipationStatus() == false)
        {
            return (Environment.getAbsenteeismDays() + Environment.getPresenteeismDays());
        }
        else return 0.0;

    }//end analysis_GetAbsences_NonParticipating()
    
    /**
     * Computes the annual average absence due to diabetes
     * for all employees.
     *  
     * @return the annual absence due to diabetes
     * for all employees.
     */
    public double analysis_GetAbsences_Average_Total()
    {

        if (analysis_GetEmployeeCount_Aggregate()== 0)
        {
            return 0.0;
        }//end if
        else
        {
            if (getCurrentDiseaseStatus() == Disease.Diabetes)
            {
                return (Environment.getAbsenteeismDays() + Environment.getPresenteeismDays())/
                    analysis_GetEmployeeCount_Aggregate();
            }
            else return 0.0;
        }//end else

    }//end analysis_GetAbsences_Average_Participating()
    
    /**
     * Computes the annual average absence due to diabetes
     * for participating employees.
     *  
     * @return the annual absence due to diabetes
     * for participating employees.
     */
    public double analysis_GetAbsences_Average_Participating()
    {

        if (analysis_GetParticipationCount_Aggregate()== 0)
        {
            return 0.0;
        }//end if
        else
        {
            if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                    getCurrentProgramParticipationStatus() == true)
            {
                return (Environment.getAbsenteeismDays() + Environment.getPresenteeismDays())/
                    analysis_GetParticipationCount_Aggregate();
            }
            else return 0.0;
        }//end else

    }//end analysis_GetAbsences_Average_Participating()
    
    /**
     * Computes the annual average absence due to diabetes
     * for non-participating employees.
     *  
     * @return the annual absence due to diabetes
     * for non-participating employees.
     */
    public double analysis_GetAbsences_Average_NonParticipating()
    {

        if (analysis_GetEmployeeCount_Aggregate() - analysis_GetParticipationCount_Aggregate()== 0)
        {
            return 0.0;
        }//end if
        else
        {
            if (getCurrentDiseaseStatus() == Disease.Diabetes &&
                    getCurrentProgramParticipationStatus() == false)
            {
                return (Environment.getAbsenteeismDays() + Environment.getPresenteeismDays())/
                    (analysis_GetEmployeeCount_Aggregate() - analysis_GetParticipationCount_Aggregate());
            }
            else return 0.0;
        }//end else

    }//end analysis_GetAbsences_Average_NonParticipating()
    
    /**
     * Computes the total annual program costs 
     * for participating employees.
     *  
     * @return the total annual program costs 
     * for participating employees.
     */
    public double analysis_GetProgramCosts_Total()
    {

        if (getCurrentProgramParticipationStatus() == true)
        {
            return (analysis_DetermineProgramCost_Base() +
                    analysis_DetermineProgramCost_Incentive() +
                    analysis_DetermineProgramCost_ChoiceArchitecture());
        }
        else return 0.0;

    }//end analysis_GetAnnualProgramCosts_Total()
    

    /**
     * Computes the base design annual program costs 
     * for participating employees.
     *  
     * @return the base design annual program costs 
     * for participating employees.
     */
    public double analysis_GetProgramCosts_Base()
    {

        if (getCurrentProgramParticipationStatus() == true)
        {
            return analysis_DetermineProgramCost_Base();
        }
        else return 0.0;

    }//end analysis_GetProgramCosts_Base()
 
    /**
     * Computes the incentive annual program costs 
     * for participating employees.
     *  
     * @return the incentive annual program costs 
     * for participating employees.
     */
    public double analysis_GetProgramCosts_Incentive()
    {

        if (getCurrentProgramParticipationStatus() == true)
        {
            return analysis_DetermineProgramCost_Incentive();
        }
        else return 0.0;

    }//end analysis_GetProgramCosts_Incentive()
 
    /**
     * Computes the choice architecture design annual program costs 
     * for participating employees.
     *  
     * @return the choice architecture annual program costs 
     * for participating employees.
     */
    public double analysis_GetProgramCosts_ChoiceArchitecture()
    {

        if (getCurrentProgramParticipationStatus() == true)
        {
            return analysis_DetermineProgramCost_ChoiceArchitecture();
        }
        else return 0.0;

    }//end analysis_GetProgramCosts_ChoiceArchitecture()
 
    
    /**
     * Computes the base design annual program cost
     * for each Employee.
     *  
     * @return the base design annual program cost 
     * for each Employee.
     */
    public double analysis_DetermineProgramCost_Base()
    {

        double baseProgramCost = 0.0;
        
        switch (Environment.getProgramDesign_Type())
        {
            case None: 
            {
                baseProgramCost = 0.0;
                break;
            }
            
            case Level1:
            {
                baseProgramCost = Environment.getProgramDesign_TypeLevel1AnnualCost();
                break;
            }
            
            case Level2:
            {
                baseProgramCost = Environment.getProgramDesign_TypeLevel2AnnualCost();
                break;
            }
        }//end switch
        
         
         return (baseProgramCost);

    }//end analysis_DetermineProgramCost_Base()
    
    /**
     * Computes the incentive annual program cost
     * for each Employee.
     *  
     * @return the incentive annual program cost 
     * for each Employee.
     */
    public double analysis_DetermineProgramCost_Incentive()
    {

        double incentiveCost = 0.0;
        
         switch (Environment.getProgramDesign_IncentiveIntensity())
        {
            case None: 
            {
                incentiveCost = 0.0;
                break;
            }
            
            case Level1:
            {
                incentiveCost = Environment.getProgramDesign_IncentiveLevel1AnnualCost();
                break;
            }
            
            case Level2:
            {
                incentiveCost = Environment.getProgramDesign_IncentiveLevel2AnnualCost();
                break;
            }
         }//end switch
         
         return (incentiveCost);

    }//end analysis_GetProgramCost_Incentive()
    
    /**
     * Computes the choice architecture program cost
     * for each Employee.
     *  
     * @return the choice architecture program cost 
     * for each Employee.
     */
    public double analysis_DetermineProgramCost_ChoiceArchitecture()
    {

        double choiceArchitectureCost = 0.0;
        
         switch (Environment.getProgramDesign_ChoiceArchitectureIntensity())
        {
            case None: 
            {
                choiceArchitectureCost = 0.0;
                break;
            }
            
            case Level1:
            {
                choiceArchitectureCost = Environment.getProgramDesign_ChoiceArchitectureLevel1AnnualCost();
                break;
            }
            
            case Level2:
            {
                choiceArchitectureCost = Environment.getProgramDesign_ChoiceArchitectureLevel2AnnualCost();
                break;
            }
         }//end switch
         
         return (choiceArchitectureCost);

    }//end analysis_GetProgramCost_ChoiceArchitecture()
   
//==============================================================================


//==============================================================================
// Methods:  getter and setter methods
//------------------------------------------------------------------------------

     public int getCurrentAge()
    {
        return (currentAge);
    }

    public Disease getCurrentDiseaseStatus()
    {
        return (currentDiseaseStatus);
    }

    public StageOfChange getCurrentStageOfChange()
    {
        return (currentStageOfChange);
    }

    public ArrayList<Goal> getEmployeeGoals()
    {
        return employeeGoals;
    }

    public WeightCategory getCurrentWeightCategory()
    {
        return currentWeightCategory;
    }

    public boolean getPreviousProgramParticipationStatus()
    {
        return previousProgramParticipationStatus;
    }

    private StageOfChange getPreviousStageOfChange()
    {
        return previousStageOfChange;
    }
    
    private GoalDescription getCurrentEmployeeFirstGoal()
    {
        return currentEmployeeFirstGoal;
    }
    
    private boolean getPreviousEmploymentStatus()
    {
        return previousEmploymentStatus;
    }
    
    private boolean getPreviousRetirementStatus()
    {
        return previousRetirementStatus;
    }
    
    private int getEmploymentTerminationAge()
    {
        return employmentTerminationAge;
    }
    
    private int getEmploymentEntryAge()
    {
        return employmentEntryAge;
    }
    
    
//==============================================================================
    
}//end Employee class


